{% extends 'is_core/base.html' %}
{% load i18n %}

{% block content %}
  <h2 class="background-serialization">{% trans 'The data are exported on background' %}</h2>
  <p>{% trans 'You will receive E-mail with a link to exported data after its completion' %}</p>
{% endblock %}
